package info;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.ContentSummary;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.security.UserGroupInformation;



public class RemoveListCDPFromDEV {
	public void start() throws Exception {
		System.setProperty("HADOOP_USER_NAME", "hdfs");
		
		Connection conn;
		String sys_nm;
		
		List<String> cdh_tables = new ArrayList<>();
		List<String> cdp_tables = new ArrayList<>();
		
		sys_nm = "cdh_dev";
		conn = DriverManager.getConnection("jdbc:mysql://ip:port/metastore?userSSL=false","id","pw");
		
		List<String> tables_cdh = getTableInfo(sys_nm, conn);
		getHdfsInfo(sys_nm, tables_cdh);
		
		cdh_tables.addAll(tables_cdh);
		System.out.println("get cdh_tables");
		
		
		sys_nm = "cdp";
		conn = DriverManager.getConnection("jdbc:mysql://ip:port/metastore?useSSL=false","id","pw");
		List<String> tables_cdp = getTableInfo(sys_nm, conn);
		getHdfsInfo(sys_nm, tables_cdp);
		
		cdp_tables.addAll(tables_cdp);
		System.out.println("get cdp_tables");
		
		conn = DriverManager.getConnection(
				"jdbc:impala://ip:port/default;UseNativeQuery=1","id","pw");
		
		Statement stmt = conn.createStatement();
		stmt.execute("delete from default.remove_list_cdp");
		
		System.out.println("delete from default.remove_list_cdp");
		
		PreparedStatement pstmt = conn.prepareStatement(
				"upsert into default.remove_list_cdp(sys_id, db_nm, tbl_nm, create_time, compare_db_tbl, db_tbl) " +
		        "values (?,?,?,?,?,?)");
		
		/*
		 * 1.sys,2.db,3.tbl,4.create_time,5.compare_db_tbl,6.db_tbl,7.compare_owner,8.cdh_owner,9.cdp_owner
		 * 10.compare_type,11.cdh_type,12.cdp_type,13.compare_comments,14.cdh_comments,15.cdp_comments
		 * 16.compare_cloumn_s,17.cdh_column_s,18.cdp_column_s,19.compare_part_s,20.cdh_part_s,21.cdp_part_s
		 * 22.compare_loc,23.cdh_loc,24.cdp_loc,25.compare_len,26.cdh_len,27.cdp_len
		 */
		
		StringBuilder drop_table_cdp_list = new StringBuilder();
		
		int rows = 0;
		try {
			for (String cdp_line : cdp_tables) {
				rows++;
				
				String db_tbl_name="nothing";
				String[] cdp_arr = cdp_line.split("\t", 16);
				String cdp_tbl_name = cdp_arr[1]+cdp_arr[2];
				
				for(String cdh_line : cdh_tables) {
					String[] cdh_arr = cdh_line.split("\t", 16);
					
					if(cdp_tbl_name.compareTo(cdp_arr[1]+cdp_arr[2])==0) {
						db_tbl_name = cdh_arr[1]+"."+cdh_arr[2];
						//System.out.println(db_tbl_name);
						
					}
					else continue;
				}
				
				/* arr[]
				 * arr[0] : sys_nm, arr[1] : db_name, arr[2] : tbl_name, arr[3] : tbl_type, arr[4] : comments, arr[5] : create_time,
				 * arr[6] : owner, arr[7] : cols, arr[8] : loc_as, arr[9] : loc_to, arr[10] : parts, arr[11] : part_cols,
				 * arr[12] : part_cnt, arr[13] : dirs, arr[14] : files, arr[15] : length
				 */
				
				/*
				 * 1.sys,2.db,3.tbl,4.create_time,5.compare_db_tbl,6.db_tbl,7.compare_owner,8.hdp_owner,9.cdp_owner
				 * 10.compare_type,11.hdp_type,12.cdp_type,13.compare_comments,14.hdp_comments,15.cdp_comments
				 * 16.compare_cloumn_s,17.hdp_column_s,18.cdp_column_s,19.compare_part_s,20.hdp_part_s,21.cdp_part_s
				 * 22.compare_loc,23.hdp_loc,24.cdp_loc,25.compare_len,26.hdp_len,27.cdp_len
				 */
				
				pstmt.setString(1, cdp_arr[0]);
				pstmt.setString(2, cdp_arr[1]);
				pstmt.setString(3, cdp_arr[2]);
				pstmt.setString(4, cdp_arr[5]);
				
				if(db_tbl_name.compareTo("nothing")==0) {
					pstmt.setString(5, "X");
					db_tbl_name ="";
					
					drop_table_cdp_list.append("drop table if exists " + cdp_arr[1]+"."+cdp_arr[2]).append("/n");
					
				} else pstmt.setString(5, "O");
				
				pstmt.setString(6, cdp_arr[1]+"."+cdp_arr[2]);

				pstmt.addBatch();
				
				if((rows % 100) == 0) {
					pstmt.executeBatch();
				}
			}
			if(rows % 100 != 0 ) {
				pstmt.executeBatch();
			}
		} finally {
			stmt.close();
			pstmt.close();
			conn.close();
		}
		
		System.out.printf("%, d rows%n", rows);
		
		
		System.out.println("create file : drop_table_list");
		
		File wfile01 = new File("script/cdp/drop_table_cdp_list");
		try {
			wfile01.delete();
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(wfile01, true), "utf-8"));
			writer.write(drop_table_cdp_list.toString());
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
		

		
	private Map<String, String> loadComments() throws Exception{
		Map<String, String> map = new HashMap<>();
		
		String line;
		BufferedReader br = new BufferedReader(
				new InputStreamReader(getClass().getResourceAsStream("comments"), "UTF-8"));
		
		while ((line = br.readLine()) != null) {
			String[] arr = line.split("\t", 3);
			map.put(arr[0] + "."+arr[1], arr[2]);
		}
		return map;
	}
		
	
	private void getHdfsInfo(String sys_nm, List<String> tables) throws Exception {
		Configuration conf = new Configuration();
		if(sys_nm.equals("cdh_dev")) {
			conf.addResource(new Path("conf/cdh_dev/core-site.xml"));
			conf.addResource(new Path("conf/cdh_dev/hdfs-site.xml"));
			conf.setBoolean("fs.hdfs.impl.disable.cache",true);
		}
		else if(sys_nm.equals("cdp")) {
			conf.addResource(new Path("conf/cdp_prd/core-site.xml"));
			conf.addResource(new Path("conf/cdp_prd/hdfs-site.xml"));
			conf.setBoolean("fs.hdfs.impl.disable.cache",true);
		}
		else {
			throw new RuntimeException("sys_nm is (cdh_dev, cdp)");
		}
		
		FileSystem hdfs = FileSystem.get(conf);
		try {
			for (int i=0; i<tables.size(); i++) {
				String line = tables.get(i);
				String[] arr = line.split("\t", 13);
				String loc_as = arr[8];
				//System.out.println(loc_as);
			
				try {
					ContentSummary summary = hdfs.getContentSummary(new Path(loc_as));
					
					long dirs = summary.getDirectoryCount();
					long files = summary.getFileCount();
					long length = summary.getLength();
					
					line += String.format("\t%s\t%s\t%s", dirs, files, length);		
				}
				catch (FileNotFoundException | IllegalArgumentException e) {
					//line += "\t\t\t";
					line += String.format("\t%s\t%s\t%s", "", "", "");
				}
				finally {
					System.out.println(i+1 + " "+ line);
					tables.set(i, line);
				}
			}
		} finally {
			hdfs.close();
		}
	}
	
	private List<String> getTableInfo(String sys_nm, Connection conn) throws Exception {
		//System.out.println("getTableInfo : " + sys_nm);
		
		Map<String, String> map;
		if(sys_nm.equals("cdh_dev")) {
			map = null;
		}
		else if (sys_nm.equals("cdp")) {
			map = null;
		}
		else {
			throw new RuntimeException("sys_nm is (cdh_dev, cdp)");
		}
		
		List<String> tables = new ArrayList<>();
		
		try {
			Statement stmt = conn.createStatement();
			
			ResultSet rs = stmt.executeQuery(
					"select a.NAME, b.TBL_NAME, b.TBL_TYPE,\n" +
							"(select PARAM_VALUE from metastore.TABLE_PARAMS where TBL_ID = b.TBL_ID and PARAM_KEY='comment') as COMMENTS, \n" +
							"from_unixtime(b.CREATE_TIME) as CREATE_TIME, b.OWNER,\n" +
							"(select count(1) from metastore.COLUMNS_V2 where CD_ID = c.CD_ID) as COLUMNS,\n" +
							"c.LOCATION as LOC_AS,\n" +
							"concat('/data', '/', a.NAME, '/', b.TBL_NAME) as LOC_TO,\n" +
							"(select count(1) from metastore.PARTITION_KEYS where TBL_ID = b.TBL_ID) as PARTS,\n" +
							"(select group_concat(PKEY_NAME) from metastore.PARTITION_KEYS where TBL_ID = b.TBL_ID) as PART_COLS,\n" +
							"(select count(1) from metastore.PARTITIONS where TBL_ID = b.TBL_ID) as PART_CNT\n" +
							"from metastore.DBS a\n"+
							"join metastore.TBLS b\n"+
							" on b.DB_ID = a.DB_ID\n" +
							"join metastore.SDS c\n" +
							" on c.SD_ID = b.SD_ID\n" +
							"where a.NAME not in ('sys', 'information_schema','default')\n" +
							//"where a.NAME in ('test')\n" +
							"order by 1, 2\n"
					);
			while (rs.next()) {
				String db_name = rs.getString("NAME");
				String tbl_name = rs.getString("TBL_NAME");
				String tbl_type = rs.getString("TBL_TYPE");
				String comments = rs.getString("COMMENTS");
				if(comments == null) comments ="";
				
				String create_time = rs.getString("CREATE_TIME");
				create_time = create_time.replaceAll("\\.0$", "");
				
				String owner = rs.getString("OWNER");
				String cols = rs.getString("COLUMNS");
				String loc_as = rs.getString("LOC_AS");
				String loc_to = rs.getString("LOC_TO");
				String parts = rs.getString("PARTS");
				String part_cols = rs.getString("PART_COLS");
				if(part_cols == null) part_cols ="";
				
				String part_cnt = rs.getString("PART_CNT");
				
				if(comments.isEmpty() || comments.contains("?")) {
					if(map != null) {
						String cmt = map.get(db_name +"."+ tbl_name);
						if(cmt != null) {
							comments = cmt;
						}
					}
				}
				
				String line = String.format("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s",
						sys_nm,db_name,tbl_name,tbl_type,comments,create_time,owner,cols,loc_as,loc_to,parts,part_cols,part_cnt);
				
				tables.add(line);
				
			}
			rs.close();
			stmt.close();
			
		} finally {
			conn.close();
		}
		
		System.out.println("getTableInfo : " + sys_nm);
		return tables;
	}
	
	public static void main(String[] args) throws Exception {
		long s = System.currentTimeMillis();
		
		RemoveListCDPFromDEV test = new RemoveListCDPFromDEV();
		test.start();
		
		System.out.printf("%,.3f sec%n", (System.currentTimeMillis() - s) / 1000f);
	}
		
}

