package info;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.ContentSummary;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.security.UserGroupInformation;



public class TableInfoCdhDev {
	public void start() throws Exception {
		System.setProperty("HADOOP_USER_NAME", "hdfs");
		
		Connection conn ;
		String sys_nm;
		List<String> cdh_tables = new ArrayList<>();

		sys_nm = "cdh_dev";
		conn = DriverManager.getConnection("jdbc:mysql://ip:port/metastore?userSSL=false","id","pw");
		
		List<String> tables_cdh = getTableInfo(sys_nm, conn);
		getHdfsInfo(sys_nm, tables_cdh);
		
		//System.out.println("tables_cdh.size()="+tables_cdh.size());
		cdh_tables.addAll(tables_cdh);
		
		System.out.println("get table info cdh dev");
		
		conn = DriverManager.getConnection(
				"jdbc:impala://ip:port/default;UseNativeQuery=1","id","pw");
		
		Statement stmt = conn.createStatement();
		stmt.execute("delete from default.tbl_info_cdh_dev");
		System.out.println("delete from default.tbl_info_cdh_dev");
		
		PreparedStatement pstmt = conn.prepareStatement(
				"upsert into default.tbl_info_cdh_dev (sys_id,db_nm,tbl_nm,db_tbl,comments,tbl_type,create_time,owner,column_s,loc_as,loc_to,part_s,part_col,part_cnt,dir_s,file_s,len,mig_yn) " +
		        "values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
		
		
		
		int rows = 0;
		
		try {
			for (String line : cdh_tables) {
				rows++;
				String[] arr = line.split("\t", 16);
				//System.out.println("arr[0]="+arr[0]+"arr[1]="+arr[1]+"arr[2]="+arr[2]+"arr[3]="+arr[3]+"arr[4]="+arr[4]+"arr[5]="+arr[5]+"arr[6]="+arr[6]+"arr[7]="+arr[7]+"arr[8]="+arr[8]+"arr[9]="+arr[9]+"arr[10]="+arr[10]+"arr[11]="+arr[11]+"arr[12]="+arr[12]+"arr[13]="+arr[13]+"arr[14]="+arr[14]+"arr[15]="+arr[15]);
				
				String dirs ="";
				pstmt.setString(1, arr[0]); // sys_nm
				pstmt.setString(2, arr[1]); // db_name
				pstmt.setString(3, arr[2]); // tbl_name
				pstmt.setString(4, arr[1]+"."+arr[2]); // db_tbl
				
				pstmt.setString(5, arr[4]); // tbl_type -> comments
				pstmt.setString(6, arr[3]); // comments -> tbl_type
				pstmt.setString(7, arr[5]); // create_time
				pstmt.setString(8, arr[6]); // owner
				pstmt.setDouble(9, Double.parseDouble(arr[7])); // cols
				pstmt.setString(10, arr[8]); // loc_as
				pstmt.setString(11, arr[8].replace("hdfs://SHAIDN1D:8020", "").replace("hdfs://shaidn1d.shaidev.ad:8020", "")); // loc_to
				pstmt.setDouble(12, Double.parseDouble(arr[10])); // parts
				pstmt.setString(13, arr[11]); // part_cols
				pstmt.setDouble(14, Double.parseDouble(arr[12])); // part_cnt
				if(arr[13].isEmpty()) {
					//System.out.println("arr[13] isEmpty");
					pstmt.setDouble(15, Types.NULL);
				} else pstmt.setDouble(15, Double.parseDouble(arr[13]));
				if(arr[14].isEmpty()) {
					//System.out.println("arr[14] isEmpty");
					pstmt.setDouble(16, Types.NULL);
				} else pstmt.setDouble(16, Double.parseDouble(arr[14]));
				if(arr[15].isEmpty()) {
					//System.out.println("arr[15] isEmpty");
					pstmt.setDouble(17, Types.NULL);
				} else pstmt.setDouble(17, Double.parseDouble(arr[15]));
				pstmt.setString(18, "");

				pstmt.addBatch();
				
				if((rows % 100) == 0) {
					pstmt.executeBatch();
				}
			}

			if(rows % 100 != 0 ) {
				pstmt.executeBatch();
			}
		} finally {
			stmt.close();
			pstmt.close();
			conn.close();
		}
		System.out.println("upsert into default.tbl_info_cdh_dev :: " + rows + " rows");
		//System.out.printf("%, d rows%n", rows);
	}
	
		
	private Map<String, String> loadComments() throws Exception{
		Map<String, String> map = new HashMap<>();
		
		String line;
		BufferedReader br = new BufferedReader(
				new InputStreamReader(getClass().getResourceAsStream("comments"), "UTF-8"));
		
		while ((line = br.readLine()) != null) {
			String[] arr = line.split("\t", 3);
			map.put(arr[0] + "."+arr[1], arr[2]);
		}
		return map;
	}
		
	
	private void getHdfsInfo(String sys_nm, List<String> tables) throws Exception {
		Configuration conf = new Configuration();
		if(sys_nm.equals("cdh_dev")) {
			//conf.addResource(new Path("conf/cdh_prd/core-site.xml"));
			//conf.addResource(new Path("conf/cdh_prd/hdfs-site.xml"));
			conf.addResource(new Path("conf/cdh_dev/core-site.xml"));
			conf.addResource(new Path("conf/cdh_dev/hdfs-site.xml"));
			conf.setBoolean("fs.hdfs.impl.disable.cache",true);
		}
		else if(sys_nm.equals("cdp")) {
			conf.addResource(new Path("conf/cdp_prd/core-site.xml"));
			conf.addResource(new Path("conf/cdp_prd/hdfs-site.xml"));
			conf.setBoolean("fs.hdfs.impl.disable.cache",true);
		}
		else {
			throw new RuntimeException("sys_nm is (cdh_dev, cdp)");
		}
		
		FileSystem hdfs = FileSystem.get(conf);
		
		try {
			for (int i=0; i<tables.size(); i++) {
				String line = tables.get(i);
				String[] arr = line.split("\t", 13);
				String loc_as = arr[8];
				
				loc_as = loc_as.replace("SHAIDN1D", "shaidn1d.shaidev.ad");
			
				try {
					long dirs=0;
					FileStatus[] fss = hdfs.listStatus(new Path(loc_as));
					for (FileStatus fs : fss) {
						if(fs.getPath().getName().contains("=")) {
							dirs++;
						}
					}
					//System.out.println("it="+fss.length + "\t" + dirs);
					
					ContentSummary summary = hdfs.getContentSummary(new Path(loc_as));
					
					//long dirs = summary.getDirectoryCount();
					long files = summary.getFileCount();
					long length = summary.getLength();
					
					line += String.format("\t%s\t%s\t%s", dirs, files, length);
					
					//System.out.println(line);
					
				}
				catch (FileNotFoundException | IllegalArgumentException e) {
					//System.out.println("FileNotFoundException | IllegalArgumentException");
					line += String.format("\t%s\t%s\t%s", "", "", "");
					//line += "\t\t\t";
				}
				finally {
					//System.out.println(i+1 + " "+ line);
					tables.set(i, line);
				}
			}
		} finally {
			//System.out.println("tables.size()="+tables.size());
			hdfs.close();
		}
	}
	
	private List<String> getTableInfo(String sys_nm, Connection conn) throws Exception {
		System.out.println("getTableInfo : " + sys_nm);
		
		Map<String, String> map;
		if(sys_nm.equals("cdh_dev")) {
			map = null;
			//map = loadComments();
		}
		else if (sys_nm.equals("cdp")) {
			map = null;
		}
		else {
			throw new RuntimeException("sys_nm is (cdh_dev, cdp)");
		}
		
		List<String> tables = new ArrayList<>();
		try {
			Statement stmt = conn.createStatement();
			
			ResultSet rs = stmt.executeQuery(
					"select a.NAME, b.TBL_NAME, b.TBL_TYPE,\n" +
							"(select PARAM_VALUE from metastore.TABLE_PARAMS where TBL_ID = b.TBL_ID and PARAM_KEY='comment') as COMMENTS, \n" +
							"from_unixtime(b.CREATE_TIME) as CREATE_TIME, b.OWNER,\n" +
							"(select count(1) from metastore.COLUMNS_V2 where CD_ID = c.CD_ID) as COLUMNS,\n" +
							"c.LOCATION as LOC_AS,\n" +
							"concat('/data', '/', a.NAME, '/', b.TBL_NAME) as LOC_TO,\n" +
							"(select count(1) from metastore.PARTITION_KEYS where TBL_ID = b.TBL_ID) as PARTS,\n" +
							"(select group_concat(PKEY_NAME) from metastore.PARTITION_KEYS where TBL_ID = b.TBL_ID) as PART_COLS,\n" +
							"(select count(1) from metastore.PARTITIONS where TBL_ID = b.TBL_ID) as PART_CNT\n" +
							"from metastore.DBS a\n"+
							"join metastore.TBLS b\n"+
							" on b.DB_ID = a.DB_ID\n" +
							"join metastore.SDS c\n" +
							" on c.SD_ID = b.SD_ID\n" +
							"where a.NAME not in ('sys', 'information_schema')\n" +
							//"where a.NAME in ('test')\n" +
							"order by 1, 2\n"
					);
			while (rs.next()) {
				String db_name = rs.getString("NAME");
				String tbl_name = rs.getString("TBL_NAME");
				String tbl_type = rs.getString("TBL_TYPE");
				String comments = rs.getString("COMMENTS");
				if(comments == null) comments ="";
				
				String create_time = rs.getString("CREATE_TIME");
				create_time = create_time.replaceAll("\\.0$", "");
				
				String owner = rs.getString("OWNER");
				//owner = owner.replace("/bigdata@xxxx.com", "");
				
				String cols = rs.getString("COLUMNS");
				String loc_as = rs.getString("LOC_AS");
				String loc_to = rs.getString("LOC_TO");
				String parts = rs.getString("PARTS");
				String part_cols = rs.getString("PART_COLS");
				if(part_cols == null) part_cols ="";
				
				String part_cnt = rs.getString("PART_CNT");
				
				if(comments.isEmpty() || comments.contains("?")) {
					if(map != null) {
						String cmt = map.get(db_name +"."+ tbl_name);
						if(cmt != null) {
							comments = cmt;
						}
					}
				}
				
				String line = String.format("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s",
						sys_nm,db_name,tbl_name,tbl_type,comments,create_time,owner,cols,loc_as,loc_to,parts,part_cols,part_cnt);
				
				//System.out.println(line);
				tables.add(line);
				
			}
			rs.close();
			stmt.close();
			
		} finally {
			conn.close();
		}
		
		return tables;
	}
	
	public static void main(String[] args) throws Exception {
		long s = System.currentTimeMillis();
		
		TableInfoCdhDev test = new TableInfoCdhDev();
		test.start();
		
		System.out.printf("%,.3f sec%n", (System.currentTimeMillis() - s) / 1000f);
	}
		
}

