package schema;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class CreateTableCDPDev {
	
	private Connection conn = null;
	private Statement stmt = null;
	
	private List<String> skipTblproperties = new ArrayList<>();
	{
		skipTblproperties.add("'COLUMN_STATS_ACCURATE'");
		skipTblproperties.add("'DO_NOT_UPDATE_STATS'");
		skipTblproperties.add("'STATS_GENERATED'");
		skipTblproperties.add("'STATS_GENERATED_VIA_STATS_TASK'");
		skipTblproperties.add("'SAS OS Name'");
		skipTblproperties.add("'SAS Version'");
		skipTblproperties.add("'impala.lastComputeStatsTime'");
		skipTblproperties.add("'last_modified_by'");
		skipTblproperties.add("'last_modified_time'");
		skipTblproperties.add("'numFiles'");
		skipTblproperties.add("'numRows'");
		skipTblproperties.add("'rawDataSize'");
		skipTblproperties.add("'serialization.nul.format'");
		skipTblproperties.add("'totalSize'");
		skipTblproperties.add("'transient_lastDdlTime'");
	}
	
	public CreateTableCDPDev() throws Exception{
		//System.setProperty("java.security.krb5.conf", "conf/bda_prd/krb5.conf");
		//System.setProperty("java.security.auth.login.config", "conf/bda_prd/hive-jaas.conf");
		//System.setProperty("javax.security.auth.useSubjectCredsOnly", "false");
		
		Class.forName("org.apache.hive.jdbc.HiveDriver");
		conn = DriverManager.getConnection("jdbc:hive2://ip:port/default","id","");
		
		stmt = conn.createStatement();
		
		Runtime.getRuntime().addShutdownHook(new Thread() {
			@Override
			public void run() {
				try {
					if(stmt != null) stmt.close();
				}catch (SQLException se) {}
				try {
					if (conn != null) conn.close();
				}catch (SQLException se) {}
			}
		});
	}
	
	public void start() throws Exception {
		/*
		 * String[][] tables = {
		 * 		{"caq", "tetaapbc9"},
		 * 		{"d0", "tsmfarc39"}
		 *  };
		 */
		
		File file = new File("script/cdh_dev/create_table_list_cdp_dev");
		
		if(file.exists()) {
			BufferedReader inFile = new BufferedReader(new FileReader(file));
			String sLine = null;
			while ((sLine = inFile.readLine()) != null) {
				String[] tables = sLine.split(",");
				showCreateTable(tables[0], tables[1]);
			}
		}
		else 
			System.out.println("no file");
	}
	
	public void showCreateTable(String db, String table) throws Exception {
		
		System.out.println(db + "." + table);
		
		StringBuilder script = new StringBuilder();
		
		List<String> tblproperties = new ArrayList<>();
		boolean isTblproperties = false;
		boolean isExternal = false;
		boolean isView = false;
		
		String dropScript = "drop table if exists "+db+"."+ table +";\n";
		script.append(dropScript);
		
		ResultSet rs = stmt.executeQuery("show create table " + db + "." + table);
		while (rs.next()) {
			String line = rs.getString(1);
			System.out.println(line);
			

			line = line.replaceAll("[+$]", "");
			line = line.replace("`", "");
			
			if (line.startsWith("CREATE ")) {
				line = line.replace("(",  " (");
				
				if(line.contains(" EXTERNAL ")) {
					isExternal = true;
				}
				else if(line.contains(" VIEW ")) {
					isView = true;
					script.replace(5, 10, "view");
				
				}
			}
			else if (line.contains("org.apache.hadoop.hive.contrib.serde2.MultiDelimitSerDe")) {
				line = line.replace("contrib.", "");
			}
			else if (line.contains("\u001E")) {
				line = line.replace("\u001E", "\\u001E");
			}
			else if (line.startsWith("TBLPROPERTIES")) {
				isTblproperties = true;
				continue;
			}
			else if (line.contains("hdfs://")) {
				//line = line.trim();
	
				script.append(line.replace("hdfs://nameservice:port", "").replace("hdfs://nameservice:port", "")).append("\n");

				continue;
			}
			
			if(isTblproperties) {
				line = line.replaceAll("[,)$]", "");
				
				int idx = line.indexOf("=");
				String key = line.substring(2, idx);
				
				if (skipTblproperties.contains(key)) continue;
				if (!isExternal && key.equalsIgnoreCase("'EXTERNAL'")) continue;
				
				tblproperties.add(line);
				continue;
			}
			
			
			script.append(line);
			script.append("\n");
		}
		rs.close();
		
		
		if (tblproperties.size() > 0) {
			script.append("TBLPROPERTIES (\n");
			for (int i=0; i< tblproperties.size(); i++) {
				String prop = tblproperties.get(i);
				
				script.append(prop);
				if(i+1 < tblproperties.size()) {
					script.append(",\n");
					
				}
				else {
					script.append(")\n");
				}
			}
		}
		script.append(";\n");

		
		if (!isExternal && !isView ) {
			script.append("alter table ").append(db).append(".").append(table).append(" unset tblproperties('EXTERNAL');\n");
		}
		
		File wfile = new File("script/cdh_dev/tables_ddl_cdp_dev.txt");
		try {
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(wfile,true),"utf-8"));
			writer.write(script.toString());
			writer.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) throws Exception {
		CreateTableCDPDev test = new CreateTableCDPDev();
		test.start();
	}
}